"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const execa = require("execa");
const fs = require("fs");
const rimraf = require("rimraf");
const Ajv = require("ajv");
const util_1 = require("./util");
const schema_1 = require("./schema");
const ajv = new Ajv(); // options can be passed, e.g. {allErrors: true}
const ajvValidator = ajv.compile(schema_1.configSchema);
class Command {
    constructor(args) {
        this.input = args.input;
        this.args = args.flags;
        this.configPath = args.flags.configPath || path.join(process.cwd(), "projects.yml");
        this.config = this.getCommandConfig();
        this.config.baseDir = args.flags.baseDir || this.config.baseDir;
    }
    getCommandConfig() {
        if (!fs.existsSync(this.configPath)) {
            util_1.logger.error(`config file doesnt exist:\n ${this.configPath}`);
            process.exit(1);
        }
        const config = util_1.getConfig(this.configPath);
        this.validateConfig(config);
        return config;
    }
    validateConfig(config) {
        const valid = ajvValidator(config);
        if (!valid) {
            util_1.logger.error(`Invalid configuration file:`);
            util_1.logger.error(JSON.stringify(ajvValidator.errors, null, 2));
            process.exit(1);
        }
        return valid;
    }
    eachProject(fn) {
        let value = 0;
        for (const project of this.config.projects) {
            value++;
            const projectDir = path.join(this.config.baseDir, project.package);
            fn(projectDir, project, [value, this.config.projects.length]);
        }
    }
    cloneProject(remote, projectDir) {
        if (!fs.existsSync(projectDir)) {
            execa.sync("git", ["clone", remote, projectDir]);
            return;
        }
        util_1.logger.warn(`${projectDir} already exists`);
    }
    checkoutBranch(projectDir, branch) {
        return execa.sync("git", ["checkout", branch], { cwd: projectDir });
    }
    installDependencies(projectDir, project, countOf) {
        execa.sync("yarn", ["install", "--ignore-scripts"], { cwd: projectDir });
        util_1.logger.iterateInfo(`installed dependencies for ${project.package}`, countOf);
    }
    buildProject(projectDir, project, countOf) {
        if (project.lerna) {
            execa.sync("lerna", ["run", "build"], { cwd: projectDir });
            util_1.logger.iterateInfo(`built ${project.package}`, countOf);
            return;
        }
        const buildCommand = project.buildCommand || "build";
        execa.sync("yarn", [buildCommand], { cwd: projectDir });
        util_1.logger.iterateInfo(`built ${project.package}`, countOf);
        return;
    }
    linkSelf(projectDir, project, relink = false) {
        let cwd = projectDir;
        if (project.linkFrom) {
            cwd = path.join(projectDir, project.linkFrom);
        }
        if (relink) {
            execa.sync("yarn", ["unlink"], { cwd });
        }
        if (project.lerna) {
            return execa.sync("lerna", ["exec", "yarn", "link"], { cwd });
        }
        return execa.sync("yarn", ["link"], { cwd });
    }
    linkDependencies(projectDir, project, countOf) {
        if (project.links) {
            if (project.lerna) {
                execa.sync("lerna", ["exec", "--", "yarn", "link", ...project.links], { cwd: projectDir });
            }
            else {
                execa.sync("yarn", ["link", ...project.links], { cwd: projectDir });
            }
            util_1.logger.iterateInfo(`linked dependencies for ${project.package}`, countOf);
            return;
        }
    }
    linkTypes(projectDir, project) {
        const { baseDir, typeDefs } = this.config;
        if (project.types) {
            for (let type of project.types) {
                const typeSymbolicPath = path.join(projectDir, `node_modules/@types/${type}`);
                const typePath = path.join(baseDir, "@types", typeDefs[type].typesPath);
                rimraf.sync(typeSymbolicPath);
                fs.symlinkSync(typePath, typeSymbolicPath, "dir");
            }
        }
    }
    // default run command
    run() {
        util_1.logger.error(`Command ${process.argv[2]} does not exist`);
    }
}
exports.Command = Command;
